//
//  t4.1 PIN input  and image packet test
//  New CameraBB with new pinouts for wired breadboard
//  MJB  10/27/2020

#define USE_ILI9488

#include <stdint.h>
#include <Wire.h>
#include <SD.h>

#include "CSI_41.h"


uint16_t FRAME_WIDTH = 640;
uint16_t FRAME_HEIGHT = 480;

//#define FRAME_WIDTH 640l
//#define FRAME_HEIGHT 480l

//#define FB_WIDTH 640l
//#define FB_HEIGHT 480l
//#define FB_COUNT 2  // two frames in EXTMEM

//#define FRAMEBYTES (FRAME_WIDTH * FRAME_HEIGHT *2)
//#define FRAMEBUFFBYTES (FB_WIDTH *FB_HEIGHT *2)

uint8_t fb1[FRAMEBUFFBYTES] EXTMEM;
uint8_t fb2[FRAMEBUFFBYTES] EXTMEM;

uint32_t masterfreq = 24;
volatile uint32_t last_csistat, csiIrqCount;
volatile uint32_t fb1count, fb2count;
volatile uint32_t bswitcherrors;
uint32_t framecount;


#ifdef USE_ILI9488

#include <ILI9488_t3.h>
#define TFT_CS   10  // AD_B0_02
#define TFT_DC   9  // AD_B0_03
#define TFT_RST  8
#define RED   ILI9488_RED
#define GREEN ILI9488_GREEN
#define BLUE  ILI9488_BLUE
#define BLACK ILI9488_BLACK
#define CENTER ILI9488_t3::CENTER
ILI9488_t3 tft = ILI9488_t3(TFT_CS, TFT_DC, TFT_RST);

uint16_t image[640 * 480] EXTMEM; // QCIF: 176x144 X 2 bytes per pixel (RGB565)

#else

//ILI9341
#include <ILI9341_t3n.h>
#define TFT_CS   10  // AD_B0_02
#define TFT_DC   9  // AD_B0_03
#define TFT_RST  8
#define RED   ILI9341_RED
#define GREEN ILI9341_GREEN
#define BLUE  ILI9341_BLUE
#define BLACK ILI9341_BLACK
#define CENTER ILI9341_t3::CENTER

ILI9341_t3n tft = ILI9341_t3n(TFT_CS, TFT_DC, TFT_RST);
uint16_t image[320 * 240] DMAMEM;

#endif

extern "C" {
  extern  uint8_t external_psram_size;
}



const char compileTime [] = " Compiled on " __DATE__ " " __TIME__;

//const int ledpin    = 13;
const int imarkpin = 32;
const int pinCamReset = 14;

#define use_tft 1

//#define  LEDON digitalWriteFast(ledpin, HIGH); // Also marks IRQ handler timing
//#define  LEDOFF digitalWriteFast(ledpin, LOW);
//#define  LEDTOGGLE digitalToggleFast(ledpin);

//#define  IMARKHI digitalWriteFast(imarkpin, HIGH); // Also marks IRQ handler timing
//#define  IMARKLO digitalWriteFast(imarkpin, LOW);
//#define  IMARKTOGGLE digitalToggleFast(imarkpin);

#define BMPIMAGEOFFSET 66
const char bmp_header[BMPIMAGEOFFSET] PROGMEM =
{
  0x42, 0x4D, 0x36, 0x58, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x42, 0x00, 0x00, 0x00, 0x28, 0x00,
  0x00, 0x00, 0x40, 0x01, 0x00, 0x00, 0xF0, 0x00, 0x00, 0x00, 0x01, 0x00, 0x10, 0x00, 0x03, 0x00,
  0x00, 0x00, 0x00, 0x58, 0x02, 0x00, 0xC4, 0x0E, 0x00, 0x00, 0xC4, 0x0E, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF8, 0x00, 0x00, 0xE0, 0x07, 0x00, 0x00, 0x1F, 0x00,
  0x00, 0x00
};

# pragma pack (2)
typedef struct  tBMPHDR565 {
  uint16_t  bfType = 0x4d42;   //'bm';
  uint32_t  bfSize = 614466;// 614400 pixels + 66 header
  uint16_t  bfReserved1 = 0;
  uint16_t  bfReserved2 = 0;
  uint32_t  bfOffBits =  66; // 14 bytes to here
  uint32_t  biSize = 40;
  int32_t   biWidth = 640;
  int32_t   biHeight = -480;  // windows wants negative for top-down image
  int16_t   biPlanes = 1;
  uint16_t  biBitCount = 16 ;
  uint32_t  biCompression = 3;  // bitfields used
  uint32_t  biSizeImage = 614400;  // 640 * 480 * 2
  int32_t   biXPelsPerMeter = 0;
  int32_t   biYPelsPerMeter = 0;
  uint32_t  biClrUsed  = 0;
  uint32_t  biClrImportant = 0;// 54 bytes
  uint32_t  rmask = 0x0000F800;
  uint32_t  gmask = 0x000007E0;
  uint32_t  bmask = 0x0000001F;  //66 bytes
} BMPHDR565;

tBMPHDR565 fileheader;
File bmpfile;

void setup() {
  while (!Serial && millis() < 4000) ;
  Serial.begin(115200);
  Wire.begin();

  delay(100);
  //pinMode(ledpin, OUTPUT);
  pinMode(imarkpin, OUTPUT);
  pinMode(pinCamReset, OUTPUT);
  pinMode(37, OUTPUT);
  delay(10);
  memset(&fb1, 0, FRAMEBUFFBYTES * 2);

  digitalWriteFast(pinCamReset, LOW);
  delay(10);
  digitalWriteFast(pinCamReset, HIGH);  // subsequent resets via SCB
  //  delay(10);
  //  digitalWriteFast(pinCamReset, LOW);  // subsequent resets via SCB


  Serial.printf("\n\nOV7670 Camera Test 3 %s\n", compileTime);
  Serial.printf("EXT Memory size: %u\n", external_psram_size);
  cameraBegin(FRAME_WIDTH, FRAME_HEIGHT, fb1, fb2);

  //  cameraSetResFormat(RGB565, QVGA_320x240);
#ifdef USE_ILI9488
  cameraSetResFormat(VGA_640x480, RGB565);
#else
  cameraSetResFormat(QVGA_320x240, RGB565);
  // BUGBUG: other places use FRAME_WDITH and FRAME_HEIGHT so lets set them
  FRAME_WIDTH = 320;
  FRAME_HEIGHT = 240;
#endif

  if (use_tft) {
    // Start ILI9341
#ifdef USE_ILI9488
    tft.begin(16000000);
#else
    tft.begin();
#endif
    tft.setRotation(1);
    tft.fillScreen(RED);
    delay(500);
    tft.fillScreen(GREEN);
    delay(500);
    tft.fillScreen(BLUE);
    delay(500);
    tft.fillScreen(BLACK);
  }
  Serial.println("After cameraBegin(end setup)");
  Serial.println("s - Show Information");
  Serial.println("c - Print CSI Registers");
  Serial.println("r - Show Camera registers");
  Serial.println("f - Show a frame");
  Serial.println("x - Send Raw (Processing Sketch)");
  Serial.println("d - Save Image to SD Card");
  Serial.println("t - send Image to ILI9341");
  Serial.println("v - Send continuous images to ILI9341");

}

void loop() {
  static uint32_t lastfc;
  static uint32_t fc = 0;
  char ch;
  if (Serial.available()) {

    ch = Serial.read();
    Serial.printf("Command character received: %c\n", ch);
    switch (ch) {
      case 's': CMSI(); break;
      case 'c': CMCS(); break;
      case 'r': CMCR(); break;
      case 'f': CMGF(); break;
      case 'x': send_raw(); break;
      case 'd': save_image_SD(); break;
      case 't': send_tft(); break;
      case 'v': do_tft_like_video(); break;
      case 0x30:
        Serial.println(F("ACK CMD CAM start single shoot. END"));
        send_image();
        Serial.println(F("READY. END"));
        break;
      default:
        break;
    }
  }

  //  This extends frame count from 16 to 32 bits
  fc  = CSI_CSICR3 >> 16; // get the 16-bit frame count

  framecount += (fc - lastfc);
  lastfc = fc;
}

// Show information
void CMSI(void) {

  Serial.printf("\n\nOV7670 Camera Test 3 %s\n", compileTime);
  Serial.printf("buff1 at %p   buff2 at %p\n", &fb1, &fb2);
  Serial.printf("Camera frame count = %lu \n", framecount);
  Serial.printf("Base Address Change errors: %lu\n", bswitcherrors);
  Serial.printf("csistat: %08X, csiIrqCount: %lu\n", CSI_CSISR, csiIrqCount);
  Serial.printf("fb1 count: %6lu   fb2 count %6lu\n", fb1count, fb2count);
}

void CMCS(void) {  // Show CSI registers
  print_csi_registers();
}



// Show Camera Registers
void CMCR(void) {
  uint8_t regs[200];
  cameraReadAll(regs);
  cameraShowAll(regs);
}


// Get a frame of data
void send_tft(void) {
//#define DEBUG_TFT
  uint32_t imagesize, fbc;
  imagesize = (FRAME_WIDTH * FRAME_HEIGHT * 2);
  fbc = fb2count;
  Serial.println("Waiting for next FB2");
  while (fbc == fb2count); // wait until a new frame in fb2 is finished
  digitalWriteFast(37, HIGH);
  Serial.printf("FB2: %lu\n", fb2count);

  arm_dcache_delete(fb2, imagesize);

#ifdef DEBUG_TFT
  Serial.println("*** Image data ***");
  volatile uint16_t *pfbStart = (uint16_t*)fb2;
  for (volatile uint16_t *pfb = pfbStart; pfb < (pfbStart + FRAME_WIDTH * 16); pfb += FRAME_WIDTH) {
    Serial.printf("\n%08x: ", (uint32_t)pfb);
    for (uint16_t i = 0; i < 8; i++) Serial.printf("%04x ", pfb[i]);
    Serial.print("..");
    for (uint16_t i = (FRAME_WIDTH/2) - 4; i < (FRAME_WIDTH/2) + 4; i++) Serial.printf("%04x ", pfb[i]);
    Serial.print("..");
    for (uint16_t i = FRAME_WIDTH - 8; i < FRAME_WIDTH; i++) Serial.printf("%04x ", pfb[i]);
  }
  Serial.println("\n...");
  for (volatile uint16_t *pfb = pfbStart + FRAME_WIDTH * ((FRAME_HEIGHT / 2) - 8); pfb < (pfbStart + FRAME_WIDTH * (FRAME_HEIGHT / 2 + 8)); pfb += FRAME_WIDTH) {
    Serial.printf("\n%08x: ", (uint32_t)pfb);
    for (uint16_t i = 0; i < 8; i++) Serial.printf("%04x ", pfb[i]);
    Serial.print("..");
    for (uint16_t i = (FRAME_WIDTH/2) - 4; i < (FRAME_WIDTH/2) + 4; i++) Serial.printf("%04x ", pfb[i]);
    Serial.print("..");
    for (uint16_t i = FRAME_WIDTH - 8; i < FRAME_WIDTH; i++) Serial.printf("%04x ", pfb[i]);
  }
  Serial.println("\n...");

  /*
    Serial.println("\n*** Dump start at middle 256 bytes ***");
    for (uint16_t i = 0; i < 256; i++) {
      if ((i & 0x1f) == 0 ) Serial.printf("%08x: ", (uint32_t) &fb2[FRAME_WIDTH * FRAME_HEIGHT + i]);
      Serial.printf("%02x ", fb2[FRAME_WIDTH * FRAME_HEIGHT + i]);
      if ((i & 0x1f) == 0x1f ) Serial.println();
    }
  */
#endif

  // Lets extract image from center.
  // hard code first pass:
#ifdef DEBUG_TFT
  Serial.println("\n*** Dump center colors");
#endif
  uint16_t *pimage = image;
  uint16_t image_offset_y = (FRAME_HEIGHT - tft.height()) / 2;
  uint16_t image_offset_x = (FRAME_WIDTH - tft.width()) / 2;
  uint32_t tft_image_offset = ((image_offset_y * FRAME_WIDTH) + image_offset_x) * 2;
#ifdef DEBUG_TFT
  Serial.printf("Image offsets: X:%u Y: %u bytes: %lu Total: %lu\n", image_offset_x, image_offset_y, tft_image_offset, imagesize);
#endif
  uint16_t *pfb = (uint16_t*)((uint8_t*)(fb2 + tft_image_offset));  // Point to pixels.
  uint16_t *pfb_row = pfb;
  for (int row = 0; row < (int)tft.height(); row++) {
    pfb = pfb_row;
    if ((row > 110) && (row < 130)) Serial.printf("%08x: ", (uint32_t)pfb);
    for (int col = 0; col < (int)tft.width(); col++) {
      *pimage = *pfb++;
      // Lets get an idea of data coming in.  How about center 16x16 pixels.
#ifdef DEBUG_TFT
      if ((row > 110) && (row < 130) && (col > 150) && (col < 170)) Serial.printf(" %04x", *pimage);
#endif
      pimage++;
    }
    pfb_row += FRAME_WIDTH;
#ifdef DEBUG_TFT
    if ((row > 110) && (row < 130)) Serial.println();
#endif
  }

  tft.writeRect(0, 0, tft.width(), tft.height(), image);
  digitalWriteFast(37, LOW);

}

void do_tft_like_video() {
  uint32_t fbc1, fbc2;
  uint16_t *pfb;
  uint16_t image_offset_y = (FRAME_HEIGHT - tft.height()) / 2;
  uint16_t image_offset_x = (FRAME_WIDTH - tft.width()) / 2;
  uint32_t tft_image_offset = ((image_offset_y * FRAME_WIDTH) + image_offset_x) * 2;

  while (Serial.read() != -1); // get rid of any excess characters.
  Serial.println("Starting continuous tft displays (like video) enter any key to exit");
  while (!Serial.available()) {
    fbc1 = fb1count;
    fbc2 = fb2count;
    for (;;) {
      if (fbc1 != fb1count) {
        Serial.printf("FB1: %lu\n", fb1count);
        pfb = (uint16_t*)((uint8_t*)(fb1 + tft_image_offset));  // Point to pixels.
        break;
      }
      if (fbc2 != fb2count) {
        Serial.printf("FB2: %lu\n", fb2count);
        pfb = (uint16_t*)((uint8_t*)(fb2 + tft_image_offset));  // Point to pixels.
        break;
      }
    }

    // Lets extract image from center.
    // hard code first pass:
    uint16_t *pimage = image;
    //Serial.printf("Image offsets: X:%u Y: %u bytes: %lu Total: %lu\n", image_offset_x, image_offset_y, tft_image_offset, imagesize);

    uint16_t *pfb_row = pfb;
    for (int row = 0; row < (int)tft.height(); row++) {
      pfb = pfb_row;
      for (int col = 0; col < (int)tft.width(); col++) {
        *pimage = *pfb++;
        // Lets get an idea of data coming in.  How about center 16x16 pixels.
        pimage++;
      }
      pfb_row += FRAME_WIDTH;
    }
    tft.writeRect(0, 0, tft.width(), tft.height(), image);
  }
  while (Serial.read() != -1); // get rid of any excess characters.
  Serial.println("*** Exiting video mode ***");

}

// Get a frame of data
void CMGF(void) {
  uint32_t imagesize, fbc;
  imagesize = (FRAME_WIDTH * FRAME_HEIGHT * 2);
  fbc = fb2count;
  while (fbc == fb2count); // wait until a new frame in fb2 is finished
  // Don't try sending unless your program can handle 614400 bytes in a hurry!
  //Serial.write(fb2,magesize);
  Serial.printf("fb2 ready for processing %lu nytes.", imagesize);
}

void send_image() {
  uint32_t imagesize, fbc;
  imagesize = (FRAME_WIDTH * FRAME_HEIGHT * 2);
  fbc = fb2count;
  while (fbc == fb2count); // wait until a new frame in fb2 is finished


  Serial.write(0xFF);
  Serial.write(0xAA);

  for (int temp = 0; temp < BMPIMAGEOFFSET; temp++)
  {
    Serial.write(pgm_read_byte(&bmp_header[temp]));
  }
  //Serial.write((const uint8_t *)&fileheader, sizeof(fileheader));

  Serial.write(fb2, imagesize);

  Serial.write(0xBB);
  Serial.write(0xCC);
}

void send_raw() {
  uint32_t imagesize, fbc;
  imagesize = (FRAME_WIDTH * FRAME_HEIGHT * 2);
  fbc = fb2count;
  while (fbc == fb2count); // wait until a new frame in fb2 is finished
  Serial.write(fb2, imagesize);

}

void save_image_SD() {
  if (!SD.begin(BUILTIN_SDCARD)) {
    Serial.println("initialization failed!");
    //while (1){
    //    LEDON; delay(100);
    //    LEDOFF; delay(100);
    //  }
  }
  Serial.println("initialization done.");

  delay(100);

  uint32_t imagesize, fbc;
  imagesize = (FRAME_WIDTH * FRAME_HEIGHT * 2);
  fbc = fb2count;
  while (fbc == fb2count); // wait until a new frame in fb2 is finished
  // Don't try sending unless your program can handle 614400 bytes in a hurry!
  //Serial.write(fb2,magesize);
  Serial.printf("fb2 ready to save %lu bytes.", imagesize);
  Serial.println("Saving OV7670.bmp ");
  if (SD.exists("OV7670.bmp")) {
    // delete the file:
    Serial.println("Removing OV7670.bmp...");
    SD.remove("OV7670.bmp");
  }
  bmpfile = SD.open("OV7670.bmp", FILE_WRITE);
  bmpfile.write((const uint8_t *)&fileheader, sizeof(fileheader));
  bmpfile.write(fb2, imagesize);
  bmpfile.close();
  Serial.println("File saved to SD card.");
}
